
#
# Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
# All rights reserved.
#
# $Revision: 1.2 $
# $State: Exp $
#

package Stream::IO;

use Stream::DataInput;
use Stream::StringInput;
use Stream::FileInput;
use Stream::DataOutput;
use Stream::StringOutput;
use Stream::FileOutput;
use Stream::Streamable;
use Stream::DataEncoding;

1;
