
#
# Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
# All rights reserved.
#
# $Revision: 1.2 $
# $State: Exp $
#

package Math::TrulyRandom;

require Exporter;
require DynaLoader;

@ISA = qw(Exporter DynaLoader);

bootstrap Math::TrulyRandom;

1;
